# 微盛，企业管家 https://platform.wshoto.com
from lazysdk import lazyrequests
import copy


default_headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate, zstd",
        "Accept-Language": "en-US,en;q=0.5",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Host": "platform.wshoto.com",
        "Origin": "https://platform.wshoto.com",
        "Pragma": "no-cache",
        "Referer": "https://platform.wshoto.com/index/dashboard",
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "TE": "trailers",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:139.0) Gecko/20100101 Firefox/139.0",
        "x-admin-header": "1",
        "x-clientType-header": "pc",
        "x-header-host": "platform.wshoto.com",
    }


def dashboard(
        authorization: str
):
    url = "https://platform.wshoto.com/bff/index/private/pc/dashboard?saMode=SECRET"
    headers = copy.deepcopy(default_headers)
    headers["Authorization"] = authorization
    return lazyrequests.lazy_requests(
        method="POST",
        url=url,
        headers=headers
    )


from lazysdk import lazyrequests
from lazysdk import lazytime
import showlog
import hashlib
import time


def convert_data2dict(data):
    data_lines = data.split("\n")
    res = list()
    if len(data_lines) == 1:
        return
    else:
        dict_key_line = data_lines[0]
        dict_keys = dict_key_line.split("\t")
        dict_value_lines = data_lines[1: -1]
        for each_line in dict_value_lines:
            each_line_values = each_line.split("\t")
            temp_dict = dict(zip(dict_keys, each_line_values))
            res.append(temp_dict)
    return res


def convert_format(data):
    data_list = convert_data2dict(data=data)
    for each in data_list:
        # each["Offer_Id"] = each["Offer Id"]
        each["Date"] = f'{each["Date"][0:4]}-{each["Date"][4:6]}-{each["Date"][6:8]}'  # 日期
        each["Impression"] = eval(each["Impression"])
        each["Click"] = eval(each["Click"])
        each["Conversion"] = eval(each["Conversion"])
        each["Ecpm"] = eval(each["Ecpm"])
        each["Cpc"] = eval(each["Cpc"])
        each["Ctr"] = eval(each["Ctr"])
        each["Cvr"] = eval(each["Cvr"])
        each["Ivr"] = eval(each["Ivr"])
        each["Spend"] = eval(each["Spend"])
    return data_list


def make_token(api_key):
    """
    生成token算法，使用时生成
    参考：https://adv.mintegral.com/doc/cn/guide/introduction/token.html
    :param api_key:
    :return:
    """
    timestamp = int(time.time())

    # 计算时间戳的MD5哈希值
    timestamp_md5 = hashlib.md5(str(timestamp).encode()).hexdigest()

    # 将API密钥与时间戳的MD5哈希值连接，并计算最终的MD5哈希值
    token = hashlib.md5((api_key + timestamp_md5).encode()).hexdigest()
    return {"timestamp": timestamp, "token": token}


def get_report(
        api_key: str,
        access_key: str,
        timezone: str = "+8",
        start_date: str = None,
        end_date: str = None,
        time_granularity: str = "daily",
        req_type: int = 1,
        dimension_option: list = None
):
    """
    广告投放报表_进阶版
    https://adv.mintegral.com/doc/cn/guide/report/advancedPerformanceReport.html

    {'code': 201, 'msg': 'Successful reception'}
    200 => 生成数据完成，可使用 type=2 获取数据。
    201 => 接收请求成功，等待生成数据。
    202 => 数据正在生成中。
    10000 => 参数错误或权限缺失。

    :param api_key:
    :param access_key:
    :param timezone: 时区
    :param start_date: 开始日期
    :param end_date: 结束日期
    :param time_granularity:
    :return:
    """
    url = "https://ss-api.mintegral.com/api/v2/reports/data"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)

    if timezone:
        params["timezone"] = timezone
    if not start_date:
        start_date = lazytime.get_date_string(days=-1)
    if not end_date:
        end_date = lazytime.get_date_string(days=0)
    params["start_time"] = start_date
    params["end_time"] = end_date
    params["time_granularity"] = time_granularity
    params["type"] = req_type
    if not dimension_option:
        params["dimension_option"] = "Offer"
    else:
        params["dimension_option"] = ",".join(dimension_option)
    if req_type == 1:
        return lazyrequests.lazy_requests(
            method="GET",
            url=url,
            params=params
        )
    elif req_type == 2:
        return lazyrequests.lazy_requests(
            method="GET",
            url=url,
            params=params,
            return_json=False
        )


def get_report_until_success(
        api_key: str,
        access_key: str,
        timezone: str = "+0",
        start_date: str = None,
        end_date: str = None,
        time_granularity: str = "daily",
        dimension_option: list = None
):
    """
        {'code': 201, 'msg': 'Successful reception'}
        200 => 生成数据完成，可使用 type=2 获取数据。
        201 => 接收请求成功，等待生成数据。
        202 => 数据正在生成中。
        10000 => 参数错误或权限缺失。
    """
    req_type = 1
    while True:
        res = get_report(
            api_key=api_key,
            access_key=access_key,
            req_type=req_type,
            dimension_option=dimension_option,
            start_date=start_date,
            end_date=end_date,
            time_granularity=time_granularity,
            timezone=timezone
        )
        if res["code"] == 201:
            showlog.info("接收请求成功，等待生成数据。")
            lazytime.count_down(5)
        elif res["code"] == 202:
            showlog.info("数据正在生成中。")
            lazytime.count_down(5)
        elif res["code"] == 200:
            showlog.info("生成数据完成，可使用 type=2 获取数据。")
            req_type = 2
            res = get_report(
                api_key=api_key,
                access_key=access_key,
                req_type=req_type,
                dimension_option=dimension_option,
                start_date=start_date,
                end_date=end_date,
                time_granularity=time_granularity,
                timezone=timezone
            )
            return convert_format(res.text)
        elif res["code"] == 10000:
            showlog.warning(res)
            return


def get_app_name(
        package_name: str,
        api_key: str,
        access_key: str
):
    """
    通过包名获取对应的 APP 名称
    https://adv.mintegral.com/doc/cn/guide/report/acquiringAppName.html
    :param api_key:
    :param access_key:
    :param package_name: 包名
    :return:
    """
    url = "https://ss-api.mintegral.com/api/open/v1/target-apps/app-name"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)

    params["package_name"] = package_name

    return lazyrequests.lazy_requests(
            method="POST",
            url=url,
            params=params,
            data={"package_name": package_name}
        )


def update_traffic(
        api_key: str,
        access_key: str,
        offer_id: int,
        option: str,
        mtg_ids: list = None,
):
    """
    更新流量投放状态;
    广告单元里需要优化的流量 mtgid。如果 mtgid 传空，则清空黑白名单。多个用,号分隔，最多 3000 个。
    https://adv.mintegral.com/doc/cn/guide/offer/updateTraffic.html
    :param api_key:
    :param access_key:
    :param offer_id: 广告单元 ID
    :param option:
        ENABLE:表示将某个 mtgid 添加进白名单或者将某个 mtgid 从黑名单剔除
        DISABLE:表示将某个 mtgid 添加进黑名单或者将某个 mtgid 从白名单剔除。
        ALLOW_ALL:表示恢复所有 App 的投放状态，取消黑白名单的设置。
    :param mtg_ids: 广告单元里需要优化的流量 mtgid。仅option 传 "ALLOW_ALL" 时，mtgid 可传空，如果 mtgid 传空，则清空黑白名单。多个用,号分隔，最多 3000 个。
注： 该接口采取全量更新方式，请提交完整的黑白名单设置。
    :return:
    """
    url = "https://ss-api.mintegral.com/api/open/v1/offer/target"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)
    mtg_ids_new = set(mtg_ids)  # 去重复
    mtgid = ",".join(mtg_ids_new)
    return lazyrequests.lazy_requests(
            method="PUT",
            url=url,
            params=params,
            data={
                "offer_id": offer_id,
                "option": option,
                "mtgid":  mtgid
            }
        )


def get_offer(
        api_key: str,
        access_key: str,
        page: int = 1,
        limit: int = 10,
        campaign_id: str = None,
        offer_id: str = None,
        package_name: str = None,
        ext_fields: list = None
):
    """
    获取广告单元列表
    https://adv.mintegral.com/doc/cn/guide/offer/getOffer.html
    :param api_key:
    :param access_key:
    :param page: 页数
    :param limit: 每页数量，最大值：50；大于 50 时，接口只返回 50 个
    :param campaign_id: 广告 ID，多个用,隔开
    :param offer_id: 广告单元 ID，多个用,隔开
    :param package_name: 包名，支持模糊查询
    :param ext_fields: 扩展查询,target_app:该字段用于屏蔽或者定向具体的应用
    :return:
    """
    url = "https://ss-api.mintegral.com/api/open/v1/offers"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)

    if page:
        params["page"] = page
    if limit:
        params["limit"] = limit
    if campaign_id:
        params["campaign_id"] = campaign_id
    if offer_id:
        params["offer_id"] = offer_id
    if package_name:
        params["package_name"] = package_name
    if ext_fields:
        params["ext_fields"] = ",".join(ext_fields)

    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )


def get_offer_all(
        api_key: str,
        access_key: str
):
    """
    获取广告单元列表
    :param api_key:
    :param access_key:
    :return:
    """
    page = 1
    limit = 10
    offers = list()
    while True:
        showlog.info(f"正在获取第 {page} 页的数据...")
        temp_res = get_offer(
            page=page,
            limit=limit,
            api_key=api_key,
            access_key=access_key
        )
        if temp_res["code"] == 200:
            data = temp_res["data"]
            if not data:
                break
            else:
                data_list = data["list"]
                if data_list:
                    offers.extend(data_list)
                    page += 1
                    continue
                else:
                    break
        else:
            showlog.warning(temp_res)
            break
    return offers


def update_budget(
        api_key: str,
        access_key: str,
        offer_id: int,
        option: str,
        mtg_ids: list = None,
):
    """
    更新预算
    广告单元里需要优化的流量 mtgid。如果 mtgid 传空，则清空黑白名单。多个用,号分隔，最多 3000 个。
    https://adv.mintegral.com/doc/cn/guide/offer/updateBudget.html
    :param api_key:
    :param access_key:
    :param offer_id: 广告单元 ID
    :param option:
        ENABLE:表示将某个 mtgid 添加进白名单或者将某个 mtgid 从黑名单剔除
        DISABLE:表示将某个 mtgid 添加进黑名单或者将某个 mtgid 从白名单剔除。
        ALLOW_ALL:表示恢复所有 App 的投放状态，取消黑白名单的设置。
    :param mtg_ids: 广告单元里需要优化的流量 mtgid。仅option 传 "ALLOW_ALL" 时，mtgid 可传空，如果 mtgid 传空，则清空黑白名单。多个用,号分隔，最多 3000 个。
注： 该接口采取全量更新方式，请提交完整的黑白名单设置。
    :return:
    """
    url = "https://ss-api.mintegral.com/api/open/v1/offer/budget"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)

    mtgid = ",".join(mtg_ids)
    return lazyrequests.lazy_requests(
            method="PUT",
            url=url,
            params=params,
            data={
                "offer_id": offer_id,
                "option": option,
                "mtgid":  mtgid
            }
        )


def get_campaign(
        api_key: str,
        access_key: str,
        page: int = 1,
        limit: int = 50
):
    """
    获取广告
    https://adv.mintegral.com/doc/cn/guide/campaign/getCampaign.html
    :param api_key:
    :param access_key:
    :param page:
    :param limit:
    :return:
    """
    url = "https://ss-api.mintegral.com/api/open/v1/campaign"
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)
    params["page"] = page
    params["limit"] = limit
    return lazyrequests.lazy_requests(
            method="GET",
            url=url,
            params=params
        )


def get_account_balance(
        api_key: str,
        access_key: str,
):
    """
    获取账户余额（含有账户id信息）
    https://adv.mintegral.com/doc/cn/guide/account/getAccountBalance.html
    :param api_key:
    :param access_key:
    :return:
    """
    token_info = make_token(api_key=api_key)
    params = {"access-key": access_key}
    params.update(token_info)
    url = "https://ss-api.mintegral.com/api/open/v1/account/balance"
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )




from lazysdk import lazyrequests


def postback(
        callback: str,
        event_time: int,
        event_type: int,
        purchase_amount=None
):
    """
    快手回传
    :param callback:
    :param event_time: 时间戳，单位秒
    :param event_type:
    :param purchase_amount:
    :return:
    """
    # result=1上报成功
    params = {
        "event_type": event_type,
        "event_time": event_time,
    }
    if purchase_amount:
        params["purchase_amount"] = purchase_amount
    res = lazyrequests.lazy_requests(
        method="GET",
        url=callback,
        params=params
    )
    return res




from lazysdk import lazyrequests


def campaigns(token: str):
    """
    获取所有的campaign
    """
    url = "https://o.applovin.com/campaign_management/v1/campaigns"
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        headers={"Authorization": token},
        timeout=60
    )


def campaign_targets(
        token: str,
        campaign_id: str
):
    """
    获取所有的campaign的目标信息
    """
    url = f"https://o.applovin.com/campaign_management/v1/campaign_targets/{campaign_id}"
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        headers={"Authorization": token},
        timeout=60
    )




from lazysdk import lazyrequests
from lazysdk import lazytime
import xmltodict
import copy
import time


"""
官网：https://adv.mintegral.com/cn/login
"""
default_headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflated",
        "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Cookie": "",
        "Host": "ss-api.mintegral.com",
        "Origin": "https://adv.mintegral.com",
        "Pragma": "no-cache",
        "Referer": "https://adv.mintegral.com/cn/login",
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-site",
        "Sec-GPC": "1",
        "TE": "trailers",
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:132.0) Gecko/20100101 Firefox/132.0"
    }
default_timeout = 60 * 10  # MTG下载最大超时时间为10分钟


def auth(
        cookie: str
):
    """
    验证cookie是否有效
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：{'code': 200, 'msg': 'success', 'data': {}

    """
    url = "https://ss-api.mintegral.com/api/v1/auth"
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        headers=headers
    )


def options(
        cookie: str,
        query: list = None
):
    """
    获取系统的基本选项
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：
        {
            'code': 200,
            'msg': 'success',
            'data': {
                'offer': ...,
                'campaign': ...,
                ...
                }
        }
    """
    scheme = "https"
    host = "ss-api.mintegral.com"
    filename = "/api/v1/options/_batch"
    if not query:
        query = [
            "offer",
            "campaign",
            "offer-status",
            "billing-type",
            "country",
            "timezone",
            "country-region-city",
            "bid-type"
        ]
    params = {
        "query": ",".join(query)
    }
    url = f"{scheme}://{host}{filename}"
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie
    return lazyrequests.lazy_requests(
        method="GET",
        params=params,
        url=url,
        headers=headers
    )


def offers(
        cookie: str,
        offer_id: int = None,
        method: str = "GET",
        put_data: dict = None,
        page: int = 1,
        page_size: int = 50
):
    """
    获取广告单元列表
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：{'code': 200, 'msg': 'success', 'data': {}
    """
    scheme = "https"
    host = "ss-api.mintegral.com"
    if not offer_id:
        filename = "/api/v1/offers"
        params = {
            "limit": page_size,
            "page": page,
            "order": "DESC",
            "sort": "id"
        }
    else:
        filename = f"/api/v1/offers/{offer_id}"
        if method == "PUT":
            params = put_data
        else:
            params = {}

    url = f"{scheme}://{host}{filename}"
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie
    return lazyrequests.lazy_requests(
        method=method,
        params=params,
        url=url,
        headers=headers
    )


def performance(
        cookie: str,
        total: bool = False,
        export: bool = False,

        page: int = 1,
        page_size: int = 50,
        timezone: int = 8,
        start_time: str = None,
        end_time: str = None,
        show_calendar_day: int = 2,
        breakdowns: list = None,
        metrics: list = None,
        sort: str = None,
        order: str = "DESC",
        adv_campaign_id: int = None,
        promote_country_code: str = None,
        timeout: int = default_timeout,
        stream: bool = False
):
    """
    获取广告单元列表
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：{'code': 200, 'msg': 'success', 'data': {}
    :param cookie: 已经登陆的cookie
    :param total: 为True时获取总记录数
    :param export: 为True时下载表格文件
    :param page: 页码
    :param page_size: 每页数量
    :param timezone: 时区
    :param start_time: 开始日期，例如：2025-03-05
    :param end_time: 结束日期，例如：2025-03-05
    :param show_calendar_day:
    :param breakdowns: 分组依据（高级选项）：[]
        (date：天, timestamp：小时，week：周，month：月)
        adv_offer_id：广告单元,
        bid_type：Bid Type,
        country_code：国家或地区
        app：应用id(子渠道)
        package_name：包名(子渠道)
        app_name：应用名称(子渠道)
        adv_campaign_id：广告
        received_price：单价
        ad_type：广告类型
    :param metrics: 展示数据列（高级选项）：[]
        "adv_impression",  # 展示
        "adv_click",  # 点击
        "adv_install",  # 转化
        "ecpm",  # eCPM
        "ecpc",  # CPC
        "ecpi",  # eCPI
        "ctr",  # CTR
        "ivr",  # IVR
        "cvr",  # CVR
        "adv_original_money",  # 花费
        "iaa_d0_ad_revenue",  # D0 Ad Rev
        "iaa_d0_roas",  # D0 Ad Roas
        "iaa_d3_ad_revenue",  # D3 Ad Rev
        "iaa_d3_roas",  # D3 Ad Roas
        "iaa_d7_ad_revenue",  # D7 Ad Rev
        "iaa_d7_roas",  # D7 Ad Roas
        "iap_d0_ad_revenue",  # D0 IAP Rev
        "iap_d0_roas",  # D0 IAP Roas
        "iap_d3_ad_revenue",  # D3 IAP Rev
        "iap_d3_roas",  # D3 IAP Roas
        "iap_d7_ad_revenue",  # D7 IAP Rev
        "iap_d7_roas",  # D7 IAP Roas
    :param sort: [可选-排序] 排序依据（高级选项）：adv_install：转化
    :param order:
    :param adv_campaign_id: [可选-筛选]广告名称
    :param promote_country_code: [可选-筛选]投放区域
    :param timeout: 超时时间
    :param stream: 为True时为流式下载
    """
    scheme = "https"
    host = "ss-api.mintegral.com"
    filename = "/api/v1/reports/performance"  # 获取详细数据
    filename_total = "/api/v1/reports/performance-total"  # 获取总数据数据量，该请求优先
    filename_export = "/api/v1/reports/performance-export"  # 导出数据

    if not start_time:
        start_time = lazytime.get_date_string(days=0)
    if not end_time:
        end_time = lazytime.get_date_string(days=0)
    if not breakdowns:  # 数据维度
        breakdowns = [
            "date",
            "adv_offer_id"
        ]
    if not metrics:  # 数据指标
        metrics = [
            "adv_impression",  # 展示
            "adv_click",  # 点击
            "adv_install",  # 转化
            "ecpm",  # eCPM
            "ecpc",  # CPC
            "ecpi",  # eCPI
            "ctr",  # CTR
            "ivr",  # IVR
            "cvr",  # CVR
            "adv_original_money",  # 花费
            "iaa_d0_ad_revenue",  # D0 Ad Rev
            "iaa_d0_roas",  # D0 Ad Roas
        ]
    params = {
        "limit": page_size,
        "page": page,
        "timezone": timezone,
        "start_time": start_time,
        "end_time": end_time,
        "order": order,  # 排序
        "breakdowns": ",".join(breakdowns),
        "metrics": ",".join(metrics),
        "show_calendar_day": show_calendar_day
    }
    if sort:
        params["sort"] = sort
    if adv_campaign_id:
        params["adv_campaign_id"] = adv_campaign_id
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie
    if total:
        return lazyrequests.lazy_requests(
            method="GET",
            params=params,
            url=f"{scheme}://{host}{filename_total}",
            headers=headers,
            timeout=timeout
        )
    elif export:
        params["t"] = int(time.time()*1000)
        return lazyrequests.lazy_requests(
            method="GET",
            params=params,
            url=f"{scheme}://{host}{filename_export}",
            headers=headers,
            timeout=timeout,
            return_json=False,
            stream=stream
        )  # 直接返回，由接收数据端自行处理
    else:
        return lazyrequests.lazy_requests(
            method="GET",
            params=params,
            url=f"{scheme}://{host}{filename}",
            headers=headers,
            timeout=timeout
        )


def sanitize_entities(
        xml_content
):
    """
    将非XML预定义实体全部替换
    """
    import re
    # 匹配非XML预定义实体
    from html.entities import entitydefs
    combined_map = {f'&{k};': v for k, v in entitydefs.items()}
    pattern = re.compile(r'&(?!lt;|gt;|amp;|apos;|quot;)\w+;')

    def replace_entity(match):
        entity = match.group(0)
        return combined_map.get(entity, entity)  # 未定义则保持原样

    return pattern.sub(replace_entity, xml_content)


def xml_to_dict(xml_str):
    """
    将MTG下载的表格内容转换为[dict]格式，方便后续处理
    :param xml_str:
    :return:
    """
    parsed_dict = xmltodict.parse(xml_str)

    # 提取表格数据（根据实际结构调整路径）
    rows = parsed_dict["Workbook"]["Worksheet"]["Table"]["Row"]
    headers = [cell["Data"]["#text"] for cell in rows[0]["Cell"]]
    data_list = [
        {headers[i]: cell["Data"]["#text"] for i, cell in enumerate(row["Cell"])}
        for row in rows[1:]
    ]
    return data_list


def history(
        cookie: str,
        page: int = 1,
        page_size: int = 50,
        objective_type: str = "",
        objective: list = None,
        operator: list = None,
        operation: list = None,
        feature: list = None,

        start_time: str = None,
        end_time: str = None,
        timezone: int = 8,
):
    """
    获取 操作日志
    注意：开始时间和结束时间如果为相同时间则无法获取数据
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：{'code': 200, 'msg': 'success', 'data': {}
    :param timezone: 时区，默认：UTC+8:8
    :param start_time: 开始时间，默认：days=-7，例如：2025-08-05 00:00:00
    :param end_time: 结束时间，默认：days=1，例如：2025-08-05 00:00:00
    :param objective_type: 查询类型，全部：""，广告：adv_campaign，广告单元：adv_offer
    """
    scheme = "https"
    host = "ss-api.mintegral.com"
    filename = "/api/v1/history/index"
    if not start_time:
        start_time = lazytime.get_datetime_relative(days=-7)
    if not end_time:
        end_time = lazytime.get_datetime_relative(days=1)
    if not objective:
        objective = []
    if not operator:
        operator = []
    if not operation:
        operation = []
    if not feature:
        feature = []
    data = {
        "limit": page_size,
        "page": page,
        "timezone": str(timezone),
        "start_time": start_time,
        "end_time": end_time,
        "objective_type": objective_type,
        "objective": objective,
        "operator": operator,
        "operation": operation,
        "feature": feature
    }
    url = f"{scheme}://{host}{filename}"
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie
    return lazyrequests.lazy_requests(
        method="POST",
        json=data,
        url=url,
        headers=headers
    )


def reports(
        cookie: str,
        url_filename: str,
        total: bool = False,

        page: int = 1,
        page_size: int = 50,
        timezone: int = 8,
        start_time: str = None,
        end_time: str = None,
        show_calendar_day: int = 2,
        breakdowns: list = None,
        metrics: list = None,
        sort: str = None,
        order: str = "DESC",
        adv_campaign_id: int = None,
        promote_country_code: str = None,
        timeout: int = default_timeout,
        stream: bool = False,

):
    """
    获取 报表中心报表
    需要登录：{'code': 400, 'msg': 'Please login first', 'data': None}
    验证成功：{'code': 200, 'msg': 'success', 'data': {}
    :param cookie: 已经登陆的cookie
    :param url_filename: 具体地址，例如：/api/v1/reports/creative

    :param total: 为True时获取总记录数
    :param export: 为True时下载表格文件
    :param page: 页码
    :param page_size: 每页数量
    :param timezone: 时区
    :param start_time: 开始日期，例如：2025-03-05
    :param end_time: 结束日期，例如：2025-03-05
    :param show_calendar_day:
    :param breakdowns: 分组依据（高级选项）：[]
        (date：天, timestamp：小时，week：周，month：月)
        adv_offer_id：广告单元,
        bid_type：Bid Type,
        country_code：国家或地区
        app：应用id(子渠道)
        package_name：包名(子渠道)
        app_name：应用名称(子渠道)
        adv_campaign_id：广告
        received_price：单价
        ad_type：广告类型
    :param metrics: 展示数据列（高级选项）：[]
        "adv_impression",  # 展示
        "adv_click",  # 点击
        "adv_install",  # 转化
        "ecpm",  # eCPM
        "ecpc",  # CPC
        "ecpi",  # eCPI
        "ctr",  # CTR
        "ivr",  # IVR
        "cvr",  # CVR
        "adv_original_money",  # 花费
        "iaa_d0_ad_revenue",  # D0 Ad Rev
        "iaa_d0_roas",  # D0 Ad Roas
        "iaa_d3_ad_revenue",  # D3 Ad Rev
        "iaa_d3_roas",  # D3 Ad Roas
        "iaa_d7_ad_revenue",  # D7 Ad Rev
        "iaa_d7_roas",  # D7 Ad Roas
        "iap_d0_ad_revenue",  # D0 IAP Rev
        "iap_d0_roas",  # D0 IAP Roas
        "iap_d3_ad_revenue",  # D3 IAP Rev
        "iap_d3_roas",  # D3 IAP Roas
        "iap_d7_ad_revenue",  # D7 IAP Rev
        "iap_d7_roas",  # D7 IAP Roas
    :param sort: [可选-排序] 排序依据（高级选项）：adv_install：转化
    :param order:
    :param adv_campaign_id: [可选-筛选]广告名称
    :param promote_country_code: [可选-筛选]投放区域
    :param timeout: 超时时间
    :param stream: 为True时为流式下载
    """
    scheme = "https"
    host = "ss-api.mintegral.com"

    if not start_time:
        start_time = lazytime.get_date_string(days=0)
    if not end_time:
        end_time = lazytime.get_date_string(days=0)
    if not breakdowns:  # 数据维度
        breakdowns = [
            "date",
            "adv_offer_id"
        ]
    if not metrics:  # 数据指标
        metrics = [
            "adv_impression",  # 展示
            "adv_click",  # 点击
            "adv_install",  # 转化
            "ecpm",  # eCPM
            "ecpc",  # CPC
            "ecpi",  # eCPI
            "ctr",  # CTR
            "ivr",  # IVR
            "cvr",  # CVR
            "adv_original_money",  # 花费
            "iaa_d0_ad_revenue",  # D0 Ad Rev
            "iaa_d0_roas",  # D0 Ad Roas
        ]
    params = {
        "limit": page_size,
        "page": page,
        "timezone": timezone,
        "start_time": start_time,
        "end_time": end_time,
        "order": order,  # 排序
        "breakdowns": ",".join(breakdowns),
        "metrics": ",".join(metrics),
        "show_calendar_day": show_calendar_day
    }
    if sort:
        params["sort"] = sort
    if adv_campaign_id:
        params["adv_campaign_id"] = adv_campaign_id
    headers = copy.deepcopy(default_headers)
    headers["Cookie"] = cookie

    return lazyrequests.lazy_requests(
        method="GET",
        params=params,
        url=f"{scheme}://{host}{url_filename}",
        headers=headers,
        timeout=timeout
    )



from lazysdk import lazyrequests
import showlog
import random
import time
import json


"""
2024-04-11 全面升级至3.0版本 https://developers.e.qq.com/v3.0/docs/api
"""


def make_nonce():
    """
    参考示例代码的生成一个随机数
    :return:
    """
    return str(time.time()) + str(random.randint(0, 999999))


def oauth_token2(
        app_id,
        app_secret,
        redirect_uri,
        grant_type='authorization_code',
        auth_code=None,
        refresh_token=None,
):
    """
    OAuth 2.0 授权
    获取/刷新token
    相关文档：https://developers.e.qq.com/docs/start/authorization
    :param auth_code:
    :param app_id:
    :param app_secret:
    :param grant_type: 请求的类型，可选值：authorization_code（授权码方式获取 token）、refresh_token（刷新 token）
    :param refresh_token:
    :param redirect_uri:
    :return:
    """
    redirect_uri = f'{redirect_uri}?app_id={app_id}'

    url = 'https://api.e.qq.com/oauth/token'
    params = {
        'client_id': app_id,
        'client_secret': app_secret,
        'grant_type': grant_type
    }
    if auth_code:
        params['authorization_code'] = auth_code
    if refresh_token:
        params['refresh_token'] = refresh_token
    if redirect_uri:
        params['redirect_uri'] = redirect_uri

    for k in params:
        if type(params[k]) is not str:
            params[k] = json.dumps(params[k])

    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )


def oauth_token3(
        app_id,
        app_secret,
        redirect_uri=None,
        access_token=None,
        grant_type='authorization_code',
        auth_code=None,
        refresh_token=None,
):
    """
    获取/刷新token
    相关文档：https://developers.e.qq.com/v3.0/docs/api/oauth/token
    :param access_token:
    :param auth_code:
    :param app_id:
    :param app_secret:
    :param grant_type: 请求的类型，可选值：authorization_code（授权码方式获取 token）、refresh_token（刷新 token）
    :param refresh_token:
    :param redirect_uri: 回调地址
    :return:
    """
    params = {
        'client_id': app_id,
        'client_secret': app_secret,
        'grant_type': grant_type
    }
    if access_token:
        # OAuth 相关接口无需提供 access_token、timestamp、nonce 等通用请求参数。
        params['access_token'] = access_token
        params['timestamp'] = int(time.time())
        params['nonce'] = make_nonce()
        url = 'https://api.e.qq.com/v3.0/oauth/token'
    else:
        url = 'https://api.e.qq.com/oauth/token'
    if auth_code:
        params['authorization_code'] = auth_code
    if refresh_token:
        params['refresh_token'] = refresh_token
    if redirect_uri:
        redirect_uri = f'{redirect_uri}?app_id={app_id}'
        params['redirect_uri'] = redirect_uri

    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )


def get_organization_account_relation(
        access_token: str,
        account_id=None,
        cursor=None,
        page: int = 1,
        page_size: int = 100,
        pagination_mode: str = "PAGINATION_MODE_CURSOR"
):
    """
    版本：3.0
    获取子账号列表
    这里获取子账户主要使用这个方法
    https://developers.e.qq.com/v3.0/docs/api/organization_account_relation/get
    :param access_token:
    :param account_id:
    :param cursor:
    :param page:
    :param page_size: 最小值 1，最大值 100
    :param pagination_mode: 分页方式，注意，为PAGINATION_MODE_NORMAL时，不能获取大于1000条的记录
    :return:
    """
    url = 'https://api.e.qq.com/v3.0/organization_account_relation/get'
    params = {
        'access_token': access_token,
        'timestamp': int(time.time()),
        'nonce': make_nonce(),
        "pagination_mode": pagination_mode,
        "page_size": page_size
    }
    if pagination_mode == "PAGINATION_MODE_NORMAL":
        params["page"] = page
    elif pagination_mode == "PAGINATION_MODE_CURSOR":
        params["cursor"] = cursor
    else:
        showlog.warning("pagination_mode参数错误")
        return

    if account_id:
        params["account_id"] = account_id
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )


def get_advertiser_daily_budget(
        access_token: str,
        account_id
):
    """
    版本：3.0
    获取广告账户日预算
    https://developers.e.qq.com/v3.0/docs/api/advertiser_daily_budget/get
    :param access_token:
    :param account_id:
    :return:

    应答示例
    {
        "code": 0,
        "message": "",
        "message_cn": "",
        "data": {
            "account_id": "<ACCOUNT_ID>",
            "daily_budget": 20000,
            "min_daily_budget": 10000
        }
    }
    """
    url = 'https://api.e.qq.com/v3.0/advertiser_daily_budget/get'
    params = {
        'access_token': access_token,
        'timestamp': int(time.time()),
        'nonce': make_nonce(),

        "account_id": account_id,
        "fields": [
            "account_id",
            "daily_budget"
        ]
    }
    return lazyrequests.lazy_requests(
        method="GET",
        url=url,
        params=params
    )


def update_daily_budget(
        access_token: str,
        account_id: int = None,
        daily_budget: int = None,
        update_daily_budget_spec: list = None
):
    """
    版本：3.0
    批量修改广告主日限额
    https://developers.e.qq.com/v3.0/docs/api/advertiser/update_daily_budget
    :param access_token:
    :param account_id:
    :param daily_budget: 账户预算，单位为分
    :param update_daily_budget_spec: 任务列表，[{"account_id":"aaa","daily_budget": 100},{"account_id":"bbb","daily_budget": 100}]
    :return:

    应答示例
    {
        "code": 0,
        "message": "",
        "message_cn": "",
        "data": {
            "list": [
                {
                    "code": 0,
                    "message": "",
                    "message_cn": "",
                    "account_id": "<ACCOUNT_ID>"
                }
            ],
            "fail_id_list": []
        }
    }
    """
    url = 'https://api.e.qq.com/v3.0/advertiser/update_daily_budget'
    params = {
        'access_token': access_token,
        'timestamp': int(time.time()),
        'nonce': make_nonce()
    }
    data = dict()
    if update_daily_budget_spec:
        data["update_daily_budget_spec"] = update_daily_budget_spec
    else:
        data["update_daily_budget_spec"] = [
            {
                "account_id": account_id,
                "daily_budget": daily_budget
            }
        ]
    return lazyrequests.lazy_requests(
        method="POST",
        url=url,
        params=params,
        json=data
    )
